package teen

import (
	"encoding/json"
	"fmt"
	"math/rand"
	"os"
	"server/game"
	"server/game/room"
	"server/msg"
	"server/user"
	"sort"
	"time"

	"github.com/name5566/leaf/gate"
	"github.com/name5566/leaf/log"
)

type TeenPattiRoom struct {
	Boot         string `json:"boot"`
	MinBuyin     string `json:"minBuyin"`
	ChaalLimmit  string `json:"chaalLimmit"`
	PotLimit     string `json:"potLimit"`
	TotalPlayers string `json:"totalPlayers"`
	RoomLevel    string `json:"roomLevel"`
	RoomId       string `json:"roomId"`
	Type         string `json:"type"`
}

var GameConfig struct {
	GameId    string          `json:"game_id"`
	Name      string          `json:"name"`
	Status    int             `json:"status"`
	RoomList  []TeenPattiRoom `json:"room_list"`
	ThinkTime int             `json:"think_time"`
}

// var Teen_Patti_Room = &room.Room{
// 	Id:      "teen_patti",
// 	Players: make([]*player.Player, 0),
// 	Status:  1,
// }

var SelfSitPos int32 = 4

// 定义玩家手牌的类型
type CardType int

const (
	CardTypeFlushStraight CardType = 1 // 同花顺
	CardTypeFlush         CardType = 2 // 同花
	CardTypePair          CardType = 3 // 对子
	CardTypeSingle        CardType = 4 // 散牌
)

func InitGame() {

}
func init() {
	data, err := os.ReadFile("../../bin/gamedata/teen_patti_conf.json")
	if err != nil {
		log.Fatal("%v", err)
	}
	err = json.Unmarshal(data, &GameConfig)
	if err != nil {
		log.Fatal("%v", err)
	}

	go handleEvents()
}

func createRoom(userId string, roomId string, agent gate.Agent) {
	//随即创建 1-3 个人机陪这个userId
	userData := agent.UserData().(*user.UserData)
	random := rand.Intn(3) + 1
	//先移除掉除自己之外的机器人
	userData.Teen_Patti_Room.RemoveAllRobot()
	sitPosList := randomSitPos(userData.Teen_Patti_Room, random)
	for i := 0; i < random; i++ {
		userData.Teen_Patti_Room.Players = append(userData.Teen_Patti_Room.Players, &room.Player{
			Id:        fmt.Sprintf("%d", i),
			Agent:     nil,
			IsRobot:   true,
			UserData:  createRobotUserData(),
			SitPos:    int32(sitPosList[i]),
			HandCards: &[]msg.ReqCard{},
			IsPacked:  false,
			IsSeen:    false,
			IsShow:    false,
			IsDealer:  false,
		})
	}

	randomDealer(userData.Teen_Patti_Room)
}

// 创建人机的UserData
func createRobotUserData() *user.UserData {
	return &user.UserData{
		Id:              "robot",
		Status:          1,
		Teen_Patti_Room: nil,
		Head_Image:      randomIndianHeadImage(),
		Nickname:        randomIndianName(),
	}
}

func startGame(userId string, roomId string, agent gate.Agent, teenPattiRoom *room.Room) {
	// agent := Teen_Patti_Room.GetPlayer(userId).Agent
	createRoom(userId, roomId, agent)
	agent.WriteMsg(&msg.ReqJoinRoom{
		UserId:   userId,
		RoomId:   roomId,
		GameId:   "teen_patti",
		RoomInfo: buildRoom(teenPattiRoom),
		SitPos:   SelfSitPos,
	})
	game.Module.Skeleton.AfterFunc(time.Second*2, func() {
		go startDealCard(teenPattiRoom)
	})
}

/* 完整的52张牌映射：
黑桃(0):
    i=0:  ♠A  (Color=0, Point=1)
    i=1:  ♠2  (Color=0, Point=2)
    i=2:  ♠3  (Color=0, Point=3)
    ...
    i=10: ♠J  (Color=0, Point=11)
    i=11: ♠Q  (Color=0, Point=12)
    i=12: ♠K  (Color=0, Point=13)

红桃(1):
    i=13: ♥A  (Color=1, Point=1)
    i=14: ♥2  (Color=1, Point=2)
    i=15: ♥3  (Color=1, Point=3)
    ...
    i=23: ♥J  (Color=1, Point=11)
    i=24: ♥Q  (Color=1, Point=12)
    i=25: ♥K  (Color=1, Point=13)

梅花(2):
    i=26: ♣A  (Color=2, Point=1)
    i=27: ♣2  (Color=2, Point=2)
    i=28: ♣3  (Color=2, Point=3)
    ...
    i=36: ♣J  (Color=2, Point=11)
    i=37: ♣Q  (Color=2, Point=12)
    i=38: ♣K  (Color=2, Point=13)

方块(3):
    i=39: ♦A  (Color=3, Point=1)
    i=40: ♦2  (Color=3, Point=2)
    i=41: ♦3  (Color=3, Point=3)
    ...
    i=49: ♦J  (Color=3, Point=11)
    i=50: ♦Q  (Color=3, Point=12)
    i=51: ♦K  (Color=3, Point=13)
*/
// 初始化房间的卡牌
func initCardDeck(c_room *room.Room) {
	// 创建长度为52的切片
	cards := make([]msg.ReqCard, 52)

	// 初始化卡牌
	for i := 0; i < 52; i++ {
		cards[i] = msg.ReqCard{
			Color: int32(i / 13),
			Point: int32(i%13 + 1),
		}
	}
	// 赋值给房间
	c_room.CardDeck = &cards
}

// 洗牌
func shuffleCardDeck(deck *[]msg.ReqCard) {
	rand.Shuffle(len(*deck), func(i, j int) {
		(*deck)[i], (*deck)[j] = (*deck)[j], (*deck)[i]
	})
}

// 计算一个玩家的手牌的类型，按照同花顺>同花>对子>散牌的顺序
func calculateCardType(cards []msg.ReqCard) CardType {
	// 先判断是否是同花顺
	isFlush := isFlush(cards)
	isStraight := isStraight(cards)
	if isFlush && isStraight {
		return CardTypeFlushStraight
	}
	// 判断是否是同花
	if isFlush {
		return CardTypeFlush
	}

	// 判断是否是对子
	isPair := isPair(cards)
	if isPair {
		return CardTypePair
	}

	// 散牌
	return CardTypeSingle
}

// 判断是否是同花（三张牌花色相同）
func isFlush(cards []msg.ReqCard) bool {
	if len(cards) != 3 {
		return false
	}
	return cards[0].Color == cards[1].Color && cards[1].Color == cards[2].Color
}

// 判断是否是顺子（三张牌点数连续）
func isStraight(cards []msg.ReqCard) bool {
	if len(cards) != 3 {
		return false
	}

	// 复制一个新的切片用于排序
	points := make([]int32, 3)
	for i, card := range cards {
		points[i] = card.Point
	}

	// 排序
	sort.Slice(points, func(i, j int) bool {
		return points[i] < points[j]
	})

	// 特殊情况：A,2,3
	if points[0] == 1 && points[1] == 2 && points[2] == 3 {
		return true
	}

	// 特殊情况：Q,K,A
	if points[0] == 1 && points[1] == 12 && points[2] == 13 {
		return true
	}

	// 普通情况：判断是否连续
	return points[1] == points[0]+1 && points[2] == points[1]+1
}

// 判断是否是对子（三张牌中有两张点数相同）
func isPair(cards []msg.ReqCard) bool {
	if len(cards) != 3 {
		return false
	}

	return cards[0].Point == cards[1].Point ||
		cards[1].Point == cards[2].Point ||
		cards[0].Point == cards[2].Point
}

// 计算一个玩家的手牌的总值
func calculateCardTotalValue(cards []msg.ReqCard) int {
	total := 0
	for _, card := range cards {
		total += int(card.Point)
	}
	return total
}

// 两个玩家对比手牌，如果手牌类型相同就比手牌总值，如果类型和值相同就比黑桃红桃方块梅花
func compareCards(player1 *room.Player, player2 *room.Player) int {
	cardType1 := calculateCardType(*player1.HandCards)
	cardType2 := calculateCardType(*player2.HandCards)
	if cardType1 == cardType2 {
		return compareCardValue(*player1.HandCards, *player2.HandCards)
	}
	return int(cardType1 - cardType2)
}

// 两个玩家手牌类型相同，比手牌总值
func compareCardValue(cards1 []msg.ReqCard, cards2 []msg.ReqCard) int {
	// 计算手牌总值
	cardValue1 := calculateCardTotalValue(cards1)
	cardValue2 := calculateCardTotalValue(cards2)
	if cardValue1 == cardValue2 {
		return compareCardColor(cards1, cards2)
	}
	return int(cardValue1 - cardValue2)
}

// 两个玩家手牌类型和大小都相同，比黑桃红桃方块梅花
// 返回值：
// 正数：cards1 赢
// 负数：cards2 赢
// 0：平局（理论上不会发生，因为不可能有完全相同的牌）
func compareCardColor(cards1 []msg.ReqCard, cards2 []msg.ReqCard) int {
	// 先把牌按点数排序
	sort.Slice(cards1, func(i, j int) bool {
		return cards1[i].Point < cards1[j].Point
	})
	sort.Slice(cards2, func(i, j int) bool {
		return cards2[i].Point < cards2[j].Point
	})

	// 从大牌开始比较花色
	for i := 2; i >= 0; i-- {
		if cards1[i].Color != cards2[i].Color {
			// 花色数字越小，牌越大（黑桃0 > 红桃1 > 方块2 > 梅花3）
			return int(cards2[i].Color - cards1[i].Color)
		}
	}
	return 0
}
